<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Filter\Validator\Password;

// Import framework stuff
use Org\Mxchange\CoreFramework\Filter\BaseFilter;
use Org\Mxchange\CoreFramework\Filter\Chain\FilterChainException;
use Org\Mxchange\CoreFramework\Filter\Filterable;
use Org\Mxchange\CoreFramework\Request\Requestable;
use Org\Mxchange\CoreFramework\Response\Responseable;

/**
 * A concrete filter for validating the password. This filter may intercept
 * the filter chain if no password is given or if both passwords didn't match.
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2022 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class PasswordValidatorFilter extends BaseFilter implements Filterable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this filter class
	 *
	 * @return	$filterInstance		An instance of this filter class
	 */
	public static final function createPasswordValidatorFilter () {
		// Get a new instance
		$filterInstance = new PasswordValidatorFilter();

		// Return the instance
		return $filterInstance;
	}

	/**
	 * Executes the filter with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 * @throws	FilterChainException	If this filter fails to operate
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Get passwords
		$password1 = $requestInstance->getRequestElement('pass1');
		$password2 = $requestInstance->getRequestElement('pass2');

		// Is the password still not set?
		if ((is_null($password1)) || (is_null($password2))) {
			// Not found in form so stop the filtering process
			$requestInstance->setIsRequestValid(FALSE);

			// Add a message to the response
			$responseInstance->addFatalMessage('password_unset');

			// Abort here
			throw new FilterChainException($this, self::EXCEPTION_FILTER_CHAIN_INTERCEPTED);
		} elseif ((empty($password1)) || (empty($password2))) {
			// Password is empty
			$requestInstance->setIsRequestValid(FALSE);

			// Is the password empty?
			if (empty($password1)) {
				// Add a message to the response
				$responseInstance->addFatalMessage('pass1_empty');
			}

			// Is the confirmation empty?
			if (empty($password2)) {
				// Add a message to the response
				$responseInstance->addFatalMessage('pass2_empty');
			}

			// Abort here
			throw new FilterChainException($this, self::EXCEPTION_FILTER_CHAIN_INTERCEPTED);
		} elseif ($password1 != $password2) {
			// Passwords didn't match
			$requestInstance->setIsRequestValid(FALSE);

			// Add a message to the response
			$responseInstance->addFatalMessage('pass_mismatch');

			// Abort here
			throw new FilterChainException($this, self::EXCEPTION_FILTER_CHAIN_INTERCEPTED);
		}
	}

}
