<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Debug\Output;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Debug\Debugger;
use Org\Mxchange\CoreFramework\Generic\UnsupportedOperationException;
use Org\Mxchange\CoreFramework\Output\Debug\BaseDebugOutput;
use Org\Mxchange\CoreFramework\Registry\Registerable;
use Org\Mxchange\CoreFramework\Stream\Output\OutputStreamer;

/**
 * A debug output class for the console (e.g. hub software)
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2022 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class DebugConsoleOutput extends BaseDebugOutput implements Debugger, OutputStreamer, Registerable {
	/**
	 * Cached configuration entry 'debug_*_output_timings'
	 */
	private $isDebugOutputTimingsEnabled = FALSE;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor first
		parent::__construct(__CLASS__);

		// Cache configuration entry
		$this->isDebugOutputTimingsEnabled = FrameworkBootstrap::getConfigurationInstance()->isEnabled('debug_' . FrameworkBootstrap::getRequestTypeFromSystem() . '_output_timings');
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$debugInstance	The prepared debug instance
	 */
	public static final function createDebugConsoleOutput () {
		// Get a new instance
		$debugInstance = new DebugConsoleOutput();

		// Return it
		return $debugInstance;
	}

	/**
	 * Outputs the given data without HTML tags
	 *
	 * @param	$output		The HTML'ed output
	 * @param	$stripTags	Whether HTML tags shall be stripped out
	 * @return	void
	 */
	public final function outputStream (string $output, bool $stripTags = false) {
		// Strip HTML tags out?
		if ($stripTags === true) {
			// Prepare the output without HTML tags
			$output = trim(html_entity_decode(strip_tags(stripslashes($output))));
		}

		// Are debug times enabled?
		if ($this->isDebugOutputTimingsEnabled) {
			// Output it first
			$output = $this->getPrintableExecutionTime() . $output;
		}

		// And print it out...
		printf('%s%s', str_replace('-&gt;', '->', $output), PHP_EOL);
	}

	/**
	 * Output the code
	 *
	 * @param	$outStream	Stream to output
	 * @param	$stripTags	Whether HTML tags shall be stripped out
	 * @return	void
	 */
	public final function output (string $outStream = '', bool $stripTags = false) {
		// Empty output will be silently ignored
		if (!empty($outStream)) {
			$this->outputStream($outStream, $stripTags);
		}
	}

	/**
	 * Streams the data and maybe does something to it
	 *
	 * @param	$data	The data (string mostly) to "stream"
	 * @return	$data	The data (string mostly) to "stream"
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function streamData (string $data) {
		self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('Unhandled ' . strlen($data) . ' bytes in this stream.');
		throw new UnsupportedOperationException([$this, __FUNCTION__], self::EXCEPTION_UNSPPORTED_OPERATION);
	}

}
