<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Resolver\Controller;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Controller\Controller;
use Org\Mxchange\CoreFramework\Controller\DefaultControllerException;
use Org\Mxchange\CoreFramework\Controller\InvalidControllerException;
use Org\Mxchange\CoreFramework\Factory\Object\ObjectFactory;
use Org\Mxchange\CoreFramework\Generic\FrameworkInterface;
use Org\Mxchange\CoreFramework\Helper\Application\ApplicationHelper;
use Org\Mxchange\CoreFramework\Registry\GenericRegistry;
use Org\Mxchange\CoreFramework\Resolver\BaseResolver;
use Org\Mxchange\CoreFramework\Resolver\Controller\ControllerResolver;
use Org\Mxchange\CoreFramework\Utils\Strings\StringUtils;

// Import SPL stuff
use \InvalidArgumentException;

/**
 * A generic controller resolver class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2022 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
abstract class BaseControllerResolver extends BaseResolver {
	/**
	 * Controller name
	 */
	private $controllerName = '';

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the real class
	 * @return	void
	 */
	protected function __construct (string $className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Setter for controller name
	 *
	 * @param	$controllerName	Last validated controller name
	 * @return	void
	 */
	protected final function setControllerName (string $controllerName) {
		$this->controllerName = $controllerName;
	}

	/**
	 * Getter for controller name
	 *
	 * @return	$controllerName	Last validated controller name
	 */
	protected final function getControllerName () {
		return $this->controllerName;
	}

	/**
	 * "Loads" a given controller and instances it if not yet cached. If the
	 * controller was not found one of the default controllers will be used
	 * depending on whether news shall be displayed.
	 *
	 * @param	$controllerName			A controller name we shall look for
	 * @return	$controllerInstance		A loaded controller instance
	 * @throws	InvalidControllerException	Thrown if even the requested
	 *										controller class is missing (bad!)
	 */
	protected function loadController (string $controllerName) {
		// Cache default controller
		$defaultController = FrameworkBootstrap::getConfigurationInstance()->getConfigEntry('default_' . strtolower($this->getClassPrefix()) . '_controller');

		// Create full class name
		$className = sprintf(
			'%s\%sDefaultNewsController',
			$this->getNamespace(),
			$this->getCapitalizedClassPrefix()
		);

		// Default controller
		$this->setClassName($className);

		// Generate the class name
		//* DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('BEFORE: controller=' . $controllerName);
		if ($controllerName != $defaultController) {
			// Create controller class name
			$className = sprintf(
				'%s\%s%sController',
				$this->getNamespace(),
				$this->getCapitalizedClassPrefix(),
				StringUtils::convertToClassName($controllerName)
			);

			// ... and set it
			$this->setClassName($className);
		} else {
			// No news at main controller or non-news controller
			$this->setClassName($className);
		}

		// Is this class loaded?
		//* DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('AFTER: controller=' . $this->getClassName());
		if (!class_exists($this->getClassName())) {
			// Throw an exception here
			throw new InvalidControllerException(array($this, $controllerName), self::EXCEPTION_INVALID_CONTROLLER);
		}

		// Try to read a config entry for our resolver including controller name... ;-)
		$resolverConfigEntry = sprintf('%s_cmd_%s_resolver_class', strtolower($this->getClassPrefix()), strtolower($controllerName));

		// Get the config, this will throw an exception if there is no special controller resolver
		//* DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('BASE-CONTROLLER-RESOLVER: resolverConfigEntry=%s', $resolverConfigEntry));
		$resolverClass = FrameworkBootstrap::getConfigurationInstance()->getConfigEntry($resolverConfigEntry);

		// Initiate the resolver and controller
		$resolverInstance = ObjectFactory::createObjectByConfiguredName(
			$resolverConfigEntry,
			array(
				$controllerName,
				ApplicationHelper::getSelfInstance()
			)
		);

		// Get controller instance
		$controllerInstance = ObjectFactory::createObjectByName(
			$this->getClassName(),
			array($resolverInstance)
		);

		// Return the result
		return $controllerInstance;
	}

	/**
	 * Checks whether the given controller is valid
	 *
	 * @param	$namespace			Namespace to look in, no trailing backslash
	 * @param	$controllerName		The default controller we shall execute
	 * @return	$isValid			Whether the given controller is valid
	 * @throws	InvalidArgumentException		Thrown if given controller is not set
	 * @throws	DefaultControllerException	Thrown if default controller was not found
	 */
	protected function isControllerValid (string $namespace, string $controllerName) {
		// Is a action set?
		if (empty($namespace)) {
			// Then thrown an exception here
			throw new InvalidArgumentException('Parameter "namespace" is empty', FrameworkInterface::EXCEPTION_INVALID_ARGUMENT);
		} elseif (empty($controllerName)) {
			// Then thrown an exception here
			throw new InvalidArgumentException('Parameter "controllerName" is empty', FrameworkInterface::EXCEPTION_INVALID_ARGUMENT);
		}

		// By default nothing shall be valid
		$isValid = false;

		// Create class name
		$className = sprintf(
			'%s\%s%sController',
			$namespace,
			$this->getCapitalizedClassPrefix(),
			StringUtils::convertToClassName($controllerName)
		);

		// Application's default news controller
		$appDefaultControllerName = sprintf(
			'%s\%sDefaultNewsController',
			$namespace,
			$this->getCapitalizedClassPrefix()
		);

		// Framework's default news controller
		$defaultControllerName = sprintf(
			'Org\Mxchange\CoreFramework\Controller\News\%sDefaultNewsController',
			$this->getCapitalizedClassPrefix()
		);

		// Now, let us create the full name of the controller class
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('className=%s', $className));
		$this->setClassName($className);

		// Try it hard to get an controller
		while ($isValid === false) {
			// Is this class already loaded?
			if (class_exists($this->getClassName())) {
				// This class does exist. :-)
				$isValid = true;
			} elseif ($this->getClassName() != $appDefaultControllerName) {
				// Set application's default controller
				$this->setClassName($appDefaultControllerName);
			} elseif ($this->getClassName() != $defaultControllerName) {
				// Set framework's default controller
				$this->setClassName($defaultControllerName);
			} else {
				// All is tried, give it up here
				throw new DefaultControllerException($this, self::EXCEPTION_DEFAULT_CONTROLLER_GONE);
			}
		}

		// Return the result
		return $isValid;
	}

	/**
	 * Resolves the default controller of the given controller
	 *
	 * @return	$controllerInstance		A controller instance for the default
	 *									controller
	 * @throws	InvalidControllerInstanceException	Thrown if $controllerInstance
	 *												is invalid
	 */
	public function resolveController () {
		// Init variables
		$controllerName = $this->getControllerName();

		// Get the controller
		$controllerInstance = $this->loadController($controllerName);

		// And validate it
		if ((!is_object($controllerInstance)) || (!$controllerInstance instanceof Controller)) {
			// This controller has an invalid instance!
			throw new InvalidControllerInstanceException(array($this, $controllerName), self::EXCEPTION_INVALID_CONTROLLER);
		}

		// Set last controller
		$this->setResolvedInstance($controllerInstance);

		// Return the maybe resolved instance
		return $controllerInstance;
	}

}
