<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Utils\Arrays;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Generic\FrameworkInterface;
use Org\Mxchange\CoreFramework\Object\BaseFrameworkSystem;

// Import SPL stuff
use \InvalidArgumentException;
use \OutOfBoundsException;

/**
 * A number utility class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2022 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
final class ArrayUtils extends BaseFrameworkSystem {
	/**
	 * Private constructor, no instance needed. If PHP would have a static initializer ...
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Maps numeric array keys to their corresponding associative
	 * (alpha-numberic) parts.
	 *
	 * @param	$numericArray	The array the mapping shall happen on
	 * @param	$mapping	Numeric array to map all elements from numeric keys to alpha-numeric keys
	 * @return	$mappedArray	An array with mapped values
	 * @throws	InvalidArgumentException	If a aparameter is not valid
	 */
	public static function mapNumericKeysToAssociative (array $numericArray, array $mapping) {
		// Validate parameters
		if (count($numericArray) == 0) {
			// Throw IAE
			throw new InvalidArgumentException('Array "numericArray" is empty', FrameworkInterface::EXCEPTION_INVALID_ARGUMENT);
		} elseif (count($mapping) == 0) {
			// Throw it again
			throw new InvalidArgumentException('Array "mapping" is empty', FrameworkInterface::EXCEPTION_INVALID_ARGUMENT);
		} elseif (count($numericArray) != count($mapping)) {
			// Throw it yet again
			throw new InvalidArgumentException(sprintf('numericArray()=%d does not match mapping()=%d', count($numericArray), count($mapping)));
		}

		// Init mapped array
		$mappedArray = [];

		// "Walk" over the mapping array
		foreach ($mapping as $sourceIndex => $targetName) {
			// Is the source index not there?
			if (!array_key_exists($sourceIndex, $numericArray)) {
				// Should always be there!
				throw new OutOfBoundsException(sprintf('numericArray[%d] does not exist.', $sourceIndex));
			}

			// "Map" (copy) it to new array
			$mappedArray[$targetName] = $numericArray[$sourceIndex];
		}

		// Return fully mapped array
		return $mappedArray;
	}

}
